@@ -3,10 +3,10 @@ import sys
 
 def main():
     if len(sys.argv) > 1:
-        print(1)
+        print("debug 1")
 
     else:
-        print(2)
+        print("debug 2")
 
 
 if __name__ == "__main__":
